// ************************************************************************************************
//
// Nixie-Voltmeter unter Verwendung des
//
// PIC 16F627A
//
// (x) Erfassen des Spannungswertes durch 10Bit-ADC MAX1242
//
// (x) Mittelwertbildung ber 50 Messungen
//
// (x) Ausgabe des Spannungswertes durch Nixie-Rhren: 3 x IN-14 (numerisch), 1 x IN-19B (EInheitenzeichen)
//
//
// Copyright www.jb-electronics.de
//
// ************************************************************************************************

// ************************************************************************************************
// PIC-Header einbinden
#include <pic.h>

// ************************************************************************************************
// Methodenprototypen
void sendeWert (uchar wert1, uchar wert2);
int getADC ();

// ************************************************************************************************
// Globale Variablen
static int TimeCounter, WertADC, Spannung, mean;
static uchar einer, zehner, hunderter, einheit, mean_counter;

// ************************************************************************************************
// Hauptmethode
void main (void) {

    // ********************************************************************************************
	// Konfigurationsbits setzen
	__CONFIG(0x3F2A);

	// ********************************************************************************************
	// Portspezifikationen
	#define	DATA		RB0
	#define CLK			RB1
	#define STROBE		RB2
	#define ENABLE		RB3
	#define	ADC			RB4
	#define CS			RB5
	#define CLK_ADC		RB6

	// Tristate von PORTB setzen
	TRISB = 0b0010000;

	// ********************************************************************************************
	// TIMER setzen
	T0CS = 0;
	PSA = 1;
	PS0 = 1;
	PS1 = 1;
	PS2 = 1;

	// ********************************************************************************************
	// Interrupt bei Timer-berlauf auslsen
	T0IE = 1;

	// Interrupts aktivieren
	GIE = 1;

	// Rhren einschalten
	ENABLE = 1;

	// Mittelwert ist auf 0, Spannung auch
	mean = 0;
	Spannung = 0;

	// ********************************************************************************************
	// Hauptschleife
	while (true) {

	}

}

// ************************************************************************************************
// Interrupt-Routine zum Abfragen des ADCs
static void interrupt isr (void) {

	// ********************************************************************************************
	// ein bisschen Zeit verstrichen?
	if (T0IF) {

		TimeCounter++;
		 if (TimeCounter >= 10) {

			// ADC-Wert holen
			WertADC = getADC();

			// Mittelwertberechnung
			mean_counter++;
			mean += WertADC;
			if (mean_counter >= 20) {
				Spannung = mean >> 6;
				mean = 0;
				mean_counter = 0;
			}
	
			// Errechne einzelne Stellen
			hunderter = Spannung / 100;
			zehner = Spannung / 10 - hunderter * 10;
			einer = Spannung % 10;
			einheit = 0; // 0 entspricht Volt
	
			// Wert an Schieberegister senden
			sendeWert(hunderter << 4 | zehner, einer << 4 | einheit);

			TimeCounter = 0;
		}

		// Interrupt resetten
		T0IF = 0;

	}

}

// ************************************************************************************************
// sendet die Werte 1 und 2 an die Schieberegister
void sendeWert (uchar wert1, uchar wert2) {

	// Variablen
	int Index;

	STROBE = 0;
	CLK = 0;
	for (Index = 7; Index >= 0; Index--) {
		NOP();
		if (wert1 & (1 << Index)) {
			DATA = 1;
		} else {
		 	DATA = 0;
		}
		NOP();
		CLK = 1;
		NOP();
		CLK = 0;
	}
	for (Index = 7; Index >= 0; Index--) {
		NOP();
		if (wert2 & (1 << Index)) {
			DATA = 1;
		} else {
		 	DATA = 0;
		}
		NOP();
		CLK = 1;
		NOP();
		CLK = 0;
	}
	STROBE = 1;
	NOP();
	STROBE = 0;

}

// ************************************************************************************************
// Diese Methode fragt den ADC ab
int getADC () {

	// Variablen
	int Index;
	int adc_wert = 0;

	
	// ADC auswhlen (/ChipSelect = 0)
	CS = 0;
	NOP();

	// Zehn Datenbits der Reihe nach reinholen (Start: MSB...LSB)
	for (Index = 0; Index < 10; Index++) {

		// Datenbit setzen?
		if (ADC) {
			adc_wert | = 1 << (10 - Index);
		}
		NOP();

		// positive Flanke
		CLK_ADC = 1;
		NOP();
		CLK_ADC = 0;
		NOP();

	}

	CS = 1;

	return adc_wert;

}
